import numpy as np


def calculate_r_nom_real(measurements):
    return max(measurements)


def calculate_linear_error_and_angle_range(angles, measurements, r_nom, return_data=False):
    r_points_pcnt = 5
    ABSOLUTE_R_MIN_OHM = 10
    ABSOLUTE_R_MAX_OHM = 100000
    MAX_ANGLE_RANGE = 360

    # Alignment
    x_min = np.argmin(measurements)
    h_phi = angles[1] - angles[0]
    measurements_aligned = np.concatenate((measurements[x_min:], measurements[:x_min]))

    # R nominal
    r_nom_real = calculate_r_nom_real(measurements_aligned)

    if r_nom_real > ABSOLUTE_R_MAX_OHM:
        raise RuntimeError("Полное сопротивление слишком велико: {} > 100 кОм".format(np.round(r_nom_real)))

    if r_nom_real < ABSOLUTE_R_MIN_OHM:
        raise RuntimeError("Полное сопротивление слишком мало: {} < 10 Ом".format(np.round(r_nom_real)))

    # Linear approximation
    r_range = np.max(measurements_aligned) - np.min(measurements_aligned)

    r_min_th = np.min(measurements_aligned) + r_range * r_points_pcnt / 100
    r_max_th = np.max(measurements_aligned) - r_range * r_points_pcnt / 100
    x_min_th = np.argmin(np.abs(measurements_aligned - r_min_th))
    x_max_th = np.argmin(np.abs(measurements_aligned - r_max_th))

    measurements_for_lin_approx = measurements_aligned[x_min_th: x_max_th]
    angles_for_lin_approx = np.arange(x_min_th, x_max_th) * h_phi
    angles_for_sequence_full = np.arange(len(measurements_aligned)) * h_phi

    a_matrix = np.vstack([angles_for_lin_approx, np.ones(len(angles_for_lin_approx))]).T
    k_koeff, r_offset = np.linalg.lstsq(a_matrix, measurements_for_lin_approx, rcond=None)[0]

    if k_koeff < ABSOLUTE_R_MIN_OHM / MAX_ANGLE_RANGE:
        raise RuntimeError(
            "Коэффициента наклона слишком мал: {} < {} Ом / {} град. = {}. Подробнее см. график.".format(
                np.round(k_koeff, 3), int(ABSOLUTE_R_MIN_OHM), int(MAX_ANGLE_RANGE),
                np.round(ABSOLUTE_R_MIN_OHM / MAX_ANGLE_RANGE, 3)
            )
        )

    # Working angle range
    angle_range = (r_nom_real - r_offset) / k_koeff

    # Error calculation
    x_error_calc_max = int(angle_range // h_phi)

    # Dirty hack. Throw out first point because due to finite contact size
    # some potentiometers have two points with the same minimal resistance value.
    x_error_calc_min = 1

    if x_error_calc_max < len(measurements) // 2:
        raise RuntimeError(
            "Линейный участок слишком короткий: {} < N / 2 = {} точек. Подробнее см. график.".format(
                x_error_calc_max, len(measurements) // 2
            ))

    if x_error_calc_max > len(measurements) - 1:
        raise RuntimeError("Неправильная длина линейного участка: {} >= N = {} точек. Подробнее см. график.".format(
                x_error_calc_max + 1, len(measurements)
            ))

    angles_for_error_calc = angles_for_sequence_full[x_error_calc_min:x_error_calc_max]
    measurements_for_error_calc = measurements_aligned[x_error_calc_min:x_error_calc_max]
    resistances_ref = angles_for_error_calc * k_koeff + r_offset

    resistance_errors = 100 * (measurements_for_error_calc - resistances_ref) / r_nom_real
    max_error = np.max(np.abs(resistance_errors))
    max_rel_error = max_error

    if return_data:
        return (measurements_aligned, angles_for_error_calc, resistance_errors)
    return (max_rel_error, angle_range)
